@@ -40,6 +40,7 @@ namespace Soprano {
     public:
         VirtuosoModelPrivate()
             : connectionPool( 0 ),
+              m_noStatementSignals( false ),
               m_fakeBooleanRegExp( QLatin1String("([\"'])(true|false)\\1\\^\\^(<http\\://www\\.w3\\.org/2001/XMLSchema#boolean>|\\w+\\:boolean)"),
                                    Qt::CaseInsensitive,
                                    QRegExp::RegExp2 ),
@@ -65,6 +66,7 @@ namespace Soprano {
             m_openIteratorMutex.unlock();
         }
 
+        QueryResultIterator sqlQuery( const QString& query );
         QueryResultIterator sparqlQuery( const QString& query );
 
         QString replaceFakeTypesInQuery( const QString& query );
@@ -74,6 +76,8 @@ namespace Soprano {
 
         QString m_virtuosoVersion;
 
+        bool m_noStatementSignals;
+
         VirtuosoModel* q;
 
     private:
